@@ -6,8 +6,8 @@ struct DecoderContext {
     c: ll::ZBUFFDecompressionContext,
 }
 
-impl DecoderContext {
-    fn new() -> Self {
+impl Default for DecoderContext {
+    fn default() -> Self {
         DecoderContext { c: unsafe { ll::ZBUFF_createDCtx() } }
     }
 }
@@ -38,7 +38,7 @@ pub struct Decoder<R: Read> {
 impl<R: Read> Decoder<R> {
     /// Creates a new decoder.
     pub fn new(reader: R) -> io::Result<Self> {
-        let context = DecoderContext::new();
+        let context = DecoderContext::default();
 
         try!(ll::parse_code(unsafe { ll::ZBUFF_decompressInit(context.c) }));
 
@@ -49,7 +49,7 @@ impl<R: Read> Decoder<R> {
     ///
     /// The dictionary must be the same as the one used during compression.
     pub fn with_dictionary(reader: R, dictionary: &[u8]) -> io::Result<Self> {
-        let context = DecoderContext::new();
+        let context = DecoderContext::default();
 
         try!(ll::parse_code(unsafe {
             ll::ZBUFF_decompressInitDictionary(context.c,
@@ -75,11 +75,23 @@ impl<R: Read> Decoder<R> {
     pub fn recommended_output_size() -> usize {
         unsafe { ll::ZBUFF_recommendedDOutSize() }
     }
+
+    /// Return the inner `Read`.
+    ///
+    /// Calling `finish()` is not *required* after reading a stream -
+    /// just use it if you need to get the `Read` back.
+    pub fn finish(self) -> R {
+        self.reader
+    }
 }
 
 impl<R: Read> Read for Decoder<R> {
     fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
 
+        if self.offset > self.buffer.capacity() {
+            return Ok(0); // End-of-frame reached.
+        }
+
         let mut written = 0;
         while written != buf.len() {
 
@@ -108,7 +120,7 @@ impl<R: Read> Read for Decoder<R> {
             let mut out_size = buf.len() - written;
             let mut in_size = self.buffer.len() - self.offset;
 
-            unsafe {
+            let res = unsafe {
                 let code =
                     ll::ZBUFF_decompressContinue(self.context.c,
                                                  buf[written..].as_mut_ptr(),
@@ -116,10 +128,14 @@ impl<R: Read> Read for Decoder<R> {
                                                  self.buffer[self.offset..]
                                                      .as_ptr(),
                                                  &mut in_size);
-                let _ = try!(ll::parse_code(code));
-            }
+                try!(ll::parse_code(code))
+            };
 
             written += out_size;
+            if res == 0 {
+                self.offset = self.buffer.capacity() + 1; // End-of-frame marker.
+                break;
+            }
             self.offset += in_size;
         }
         Ok(written)
